/***************************************************************************
  * Copyright (C) 2005 Ping Identity Corporation
 * All rights reserved.
 *
 * The contents of this file are the property of Ping Identity Corporation.
 * You may not copy or use this file, in either source code or executable
 * form, except in compliance with terms set by Ping Identity Corporation.
 * For further information please contact:
 *
 *      Ping Identity Corporation
 *      1099 18th St Suite 2950
 *      Denver, CO 80202
 *      303.468.2900
 *      http://www.pingidentity.com
 *
 **************************************************************************/

package com.pingidentity.pf3.adapter.idp;

import org.sourceid.saml20.adapter.conf.Configuration;
import org.sourceid.saml20.adapter.conf.Row;
import org.sourceid.saml20.adapter.conf.Table;
import org.sourceid.saml20.adapter.gui.AbstractSelectionFieldDescriptor;
import org.sourceid.saml20.adapter.gui.AdapterConfigurationGuiDescriptor;
import org.sourceid.saml20.adapter.gui.FieldDescriptor;
import org.sourceid.saml20.adapter.gui.RadioGroupFieldDescriptor;
import org.sourceid.saml20.adapter.gui.TableDescriptor;
import org.sourceid.saml20.adapter.gui.TextFieldDescriptor;
import org.sourceid.saml20.adapter.gui.validation.ConfigurationValidator;
import org.sourceid.saml20.adapter.gui.validation.ValidationException;
import org.sourceid.saml20.adapter.gui.validation.impl.RequiredFieldValidator;
import org.sourceid.saml20.adapter.idp.authn.AbstractPasswordIdpAuthnAdapter;
import org.sourceid.saml20.adapter.idp.authn.IdpAuthnAdapterDescriptor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * A authentication adapter that just has a table of valid user names and passwords.  Authentication can be
 * done via HTTP Basic or HTML form.  The choice of form or basic is admin configurable.
 * <p>
 * Not real practical for actual deployments but maybe useful for POCs and such.  Maybe a helpful example as well.
 */
public class SimplePwdAuthnAdapter extends AbstractPasswordIdpAuthnAdapter
{
    private static final String ATTR_NAME_USERNAME = "Username";
    private static final RequiredFieldValidator REQUIRED_VALIDATOR = new RequiredFieldValidator();

    private IdpAuthnAdapterDescriptor descriptor;
    private Map<String,String> userNamePassMap;
    private String realm;
    private static final String TABLE_NAME = "Users";

    private static final String FIELD_NAME_USERNAME = "User name";
    private static final String FIELD_NAME_PASSWORD = "Password";
    private static final String FIELD_NAME_REALM_NAME = "Realm Name";
    private static final String FIELD_NAME_MODE = "Operational Mode";

    public SimplePwdAuthnAdapter()
    {
        Set<String> attrContract = new HashSet<String>();
        attrContract.add(ATTR_NAME_USERNAME);
        AdapterConfigurationGuiDescriptor gui = new AdapterConfigurationGuiDescriptor();

        TextFieldDescriptor realmField = new TextFieldDescriptor(FIELD_NAME_REALM_NAME, "The HTTP Realm.");
        realmField.addValidator(REQUIRED_VALIDATOR);
        gui.addField(realmField);

        List <RadioGroupFieldDescriptor.OptionValue> options = new ArrayList<RadioGroupFieldDescriptor.OptionValue>();
        options.add(new AbstractSelectionFieldDescriptor.OptionValue("HTTP Basic Authentication", Mode.BASIC_AUTHN.toString()));
        options.add(new AbstractSelectionFieldDescriptor.OptionValue("HTML Form Authentication", Mode.FORM_AUTHN.toString()));
        RadioGroupFieldDescriptor mode = new RadioGroupFieldDescriptor(FIELD_NAME_MODE, "", options);
        mode.setDefaultValue(getOperationalMode().toString());
        mode.addValidator(REQUIRED_VALIDATOR);
        gui.addField(mode);

        TableDescriptor table = new TableDescriptor(TABLE_NAME, "A table of valid user names and passwords.");
        String description = "An individual user name for authentication";
        FieldDescriptor username = new TextFieldDescriptor(FIELD_NAME_USERNAME, description);
        username.addValidator(REQUIRED_VALIDATOR);
        table.addRowField(username);

        String description2 = "An individual password for authentication";
        FieldDescriptor password = new TextFieldDescriptor(FIELD_NAME_PASSWORD, description2, true);
        password.addValidator(REQUIRED_VALIDATOR);
        table.addRowField(password);
        gui.addTable(table);

        gui.addValidator(new ConfigurationValidator()
        {
            public void validate(Configuration configuration) throws ValidationException
            {
                Set unames = new HashSet();
                List<String> errors = new ArrayList<String>();
                Table table = configuration.getTable(TABLE_NAME);
                List<Row> rows = table.getRows();

                if (rows.isEmpty())
                {
                    throw new ValidationException("Please add at least one user to the '"+ TABLE_NAME+ "' table.");
                }

                for (Row row : rows)
                {
                    String uname = row.getFieldValue(FIELD_NAME_USERNAME);
                    if (!unames.add(uname))
                    {
                        errors.add("Duplicate user name: " + uname);
                    }
                }

                if (!errors.isEmpty())
                {
                    throw new ValidationException(errors);
                }
            }
        });

        String type = "Simple user name and password";
        descriptor = new IdpAuthnAdapterDescriptor(this, type, attrContract, false, gui, false);
    }


    protected String getRealm()
    {
        return realm;
    }

    protected Map getAuthenticationIdentifiers(String username, String password)
    {
        String pass = userNamePassMap.get(username);
        if (pass != null && pass.equals(password))
        {
            Map authenticationIdentifiers = new HashMap(2);                
            authenticationIdentifiers.put(ATTR_NAME_USERNAME, username);
            return authenticationIdentifiers;
        }
        return null;
    }

    public IdpAuthnAdapterDescriptor getAdapterDescriptor()
    {
        return descriptor;
    }

    public void configure(Configuration configuration)
    {
        Table table = configuration.getTable(TABLE_NAME);
        userNamePassMap = new HashMap<String, String>();
        for (Row row : table.getRows())
        {
             String uname = row.getFieldValue(FIELD_NAME_USERNAME);
             String pass = row.getFieldValue(FIELD_NAME_PASSWORD);
             userNamePassMap.put(uname, pass);
        }

        realm = configuration.getFieldValue(FIELD_NAME_REALM_NAME);

        String mode = configuration.getFieldValue(FIELD_NAME_MODE);
        setOperationalMode(Mode.valueOf(mode));
    }
}
