/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Web;
using System.Web.SessionState;
using Com.PingIdentity.Adapters.Pftoken.Core;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;

namespace Com.PingIdentity.PingFederate.SampleApp.SpSample.cs.main {

	/// <summary>
	/// This is the Account Link servlet of the SP Sample Application
	/// and handles local login
	/// </summary>
	public class AccountLinkingSpHandler : IHttpHandler, IRequiresSessionState {

		private Links samlLinks;

		public void ProcessRequest(HttpContext context) {
			
			HttpRequest request      = context.Request;
			HttpResponse response    = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Sp config file
			Hashtable spConfigTable;
			if(session[Constants.SP_CONFIG_TABLE] == null) {
				spConfigTable = GetSpConfigProps(context);
				session[Constants.SP_CONFIG_TABLE] = spConfigTable;
			}
			else {
				spConfigTable = (Hashtable)session[Constants.SP_CONFIG_TABLE];
			}
			context.Trace.Warn("spConfigTable has been loaded");

			samlLinks = new Links(spConfigTable, request);

			//create URL to Login Page
			String loginPage = samlLinks.getAccountLinkingSpPageUrl(Constants.GLOBAL_TYPE);

			//get resumePath
			String resumePath = request.QueryString.Get(Constants.RESUME_PATH);
			if(resumePath != null) {
				loginPage  = samlLinks.addParameter(loginPage, Constants.RESUME_PATH, resumePath);
			}
			context.Trace.Warn("resumePath = " + resumePath);

			//Get user attributes from session
			Hashtable userInfo = (Hashtable)session[Constants.USER_INFO];
			if((userInfo == null) || (userInfo.Count <= 0)) {
				response.Redirect(loginPage);
				return;
			}

			// Redirect to SP application protected page
			String mainpage = samlLinks.getSpMainPageUrl();
			if(resumePath != null) {
				resumePath = samlLinks.getResumePathUrl(resumePath);
				resumePath = setPFToken(context, resumePath, userInfo);
				session.Abandon();
				response.Redirect(resumePath);
			}
			else {
				response.Redirect(mainpage);
			}
		}

		/// <summary>
		/// setPFToken method uses the Java Integration Kit API to create an 
		/// instance of Idp Agent. The user attributes are added to this agent and 
		/// then the PFToken cookie is created as a cookie or query parameter
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <param name="url">url to append pftoken</param>
		/// <param name="userInfo">Hashtable of user attributes</param>
		/// <returns>URL with pftoken</returns>	
		private String setPFToken(HttpContext context, String url, Hashtable userInfo) 
		{
			String returnUrl = url;

			HttpRequest  request  = context.Request;
			HttpResponse response = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Sp config file
			Hashtable spConfigTable;
			if(session[Constants.SP_CONFIG_TABLE] == null) 
			{
				spConfigTable = GetSpConfigProps(context);
				session[Constants.SP_CONFIG_TABLE] = spConfigTable;
			}
			else 
			{
				spConfigTable = (Hashtable)session[Constants.SP_CONFIG_TABLE];
			}
			context.Trace.Warn("spConfigTable has been loaded");
		
			//read properties from Sp pfagent-properties file
			String spProps   = ConfigurationSettings.AppSettings["pfagent-properties"];
			String propsPath = context.Server.MapPath("config/" + spProps);

			//get password
			String password = (String)spConfigTable[Constants.PASSWORD];

			//get holder name
			String holderName = (String)spConfigTable[Constants.HOLDER_NAME];

			// Create an instance of PFTokenSpAgent
			PftokenAgent spAgent = new PftokenAgent(password,holderName);		
	
			// Add user attributes to the agent		
			spAgent.AddAttributes(userInfo);

			// Create PFToken
			returnUrl = spAgent.ConstructPFToken(request, response, url);

			return returnUrl;
		}

		/// <summary>
		/// Reads general configuration properties XML file for the
		/// SP Sample Application and converts it to Hastable object
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>Hashtable of user general configuration properties</returns>
		private Hashtable GetSpConfigProps(HttpContext context) {

			//read properties from Sp config XML file
			String spConfig  = ConfigurationSettings.AppSettings["config-file"];
			String configPath = context.Server.MapPath("config/" + spConfig);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable spConfigTable = xmlUtility.GetConfigProps(configPath);
			if(spConfigTable == null) {
				String errorMessage = "Error in " + spConfig;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			return spConfigTable;
		}

		/// <summary>
		/// Reads connection properties XML file for specifying Idp Connections
		/// and converts it to ArrayList of Hashtables with user Idp connections
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>ArrayList of user connection properties</returns>
		private ArrayList GetSpConnectionsProps(HttpContext context) {

			//read properties from Sp connections XML file
			String spConnections  = ConfigurationSettings.AppSettings["connections-file"];
			String connectionsPath = context.Server.MapPath("config/" + spConnections);

			XmlUtility xmlUtility = new XmlUtility();
			ArrayList connectionsList = xmlUtility.GetPropsFromXmlFile(connectionsPath);
			if(connectionsList == null || connectionsList.Count == 0) {
				String errorMessage = "Error in " + spConnections;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			return connectionsList;
		}

		public bool IsReusable {
			get { return true; }
		}
	}
}