/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using Com.PingIdentity.Adapters.Pftoken.Core;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;

namespace Com.PingIdentity.PingFederate.SampleApp.SpSample.cs.sso {

	/// <summary>
	/// SSOSpHandler retrieves user attributes from PFTOKEN and then creates
	/// a local session. Once a session is established, user is redirected to
	/// the SP Sample Application Protected Page.
	/// </summary>
	public class SSOSpHandler : IHttpHandler, IRequiresSessionState {

		private Links samlLinks;
		
		public void ProcessRequest(HttpContext context) {

            HttpRequest request = context.Request;
			HttpResponse response    = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Sp config file
			Hashtable spConfigTable;
			if(session[Constants.SP_CONFIG_TABLE] == null) {
				spConfigTable = GetSpConfigProps(context);
				session[Constants.SP_CONFIG_TABLE] = spConfigTable;
			}
			else {
				spConfigTable = (Hashtable)session[Constants.SP_CONFIG_TABLE];
			}
			context.Trace.Warn("spConfigTable has been loaded");

			//get properties from Sp connections file
			ArrayList spConnectionsList;
			if(session[Constants.SP_CONNECT_LIST] == null) {
				spConnectionsList = GetSpConnectionsProps(context);
				session[Constants.SP_CONNECT_LIST] = spConnectionsList;
			}
			else {
				spConnectionsList = (ArrayList)session[Constants.SP_CONNECT_LIST];
			}
			context.Trace.Warn("spConnectionsList has been loaded");

			samlLinks = new Links(spConfigTable, request);

			// Get user attributes from PFToken Sp Adapter as a Map object
			Hashtable userInfo = parsePFToken(context);
			context.Trace.Warn("userInfo has been extracted from PFToken");

			if((userInfo != null) && (userInfo.Count > 0))
			{
				// Store the attributes obtained from PFToken in session
				session[Constants.USER_INFO] = userInfo;
			}

			userInfo = (Hashtable)session[Constants.USER_INFO];

			if((userInfo != null) && (userInfo.Count > 0))
			{
				// Create the formsauth cookie
				HttpCookie authCookie = FormsAuthentication.GetAuthCookie(
					(String)userInfo[Constants.USER_ID], false);
				response.SetCookie(authCookie);
			}

			// Redirect to the User Attributes display page
			String mainpage = samlLinks.getSpMainPageUrl();
			response.Redirect(mainpage);
		}

		/// <summary>
		/// parsePFToken method uses the Java Integration Kit API to create an 
		/// instance of Sp Agent and returns the user attributes obtained from 
		/// PFToken
		/// <summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>Hashtable of user attributes</returns>
		private Hashtable parsePFToken(HttpContext context) {

			HttpRequest  request     = context.Request;
			HttpResponse response    = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Sp config file
			Hashtable spConfigTable;
			if(session[Constants.SP_CONFIG_TABLE] == null) 
			{
				spConfigTable = GetSpConfigProps(context);
				session[Constants.SP_CONFIG_TABLE] = spConfigTable;
			}
			else 
			{
				spConfigTable = (Hashtable)session[Constants.SP_CONFIG_TABLE];
			}
			context.Trace.Warn("spConfigTable has been loaded");
		
			//read properties from Sp pfagent-properties file
			String spProps   = ConfigurationSettings.AppSettings["pfagent-properties"];
			String propsPath = context.Server.MapPath("config/" + spProps);

			//get password
			String password = (String)spConfigTable[Constants.PASSWORD];

			//get holder name
			String holderName = (String)spConfigTable[Constants.HOLDER_NAME];

			// Create an instance of PFTokenSpAgent
			PftokenAgent spAgent = new PftokenAgent(password,holderName);
		
			// Get attributes from the PFTOKEN
			spAgent.ExtractFromRequest(request, response);
			Hashtable userInfo = new Hashtable(spAgent.GetAttributes());

			return userInfo;		
		}

		/// <summary>
		/// Reads general configuration properties XML file for the
		/// SP Sample Application and converts it to Hastable object
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>Hashtable of user general configuration properties</returns>
		private Hashtable GetSpConfigProps(HttpContext context) {

			//read properties from Sp config XML file
			String spConfig  = ConfigurationSettings.AppSettings["config-file"];
			String configPath = context.Server.MapPath("config/" + spConfig);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable spConfigTable = xmlUtility.GetConfigProps(configPath);
			if(spConfigTable == null) {
				String errorMessage = "Error in " + spConfig;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			return spConfigTable;
		}

		/// <summary>
		/// Reads connection properties XML file for specifying Idp Connections
		/// and converts it to ArrayList of Hashtables with user Idp connections
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>ArrayList of user connection properties</returns>
		private ArrayList GetSpConnectionsProps(HttpContext context) {

			//read properties from Sp connections XML file
			String spConnections  = ConfigurationSettings.AppSettings["connections-file"];
			String connectionsPath = context.Server.MapPath("config/" + spConnections);

			if (File.Exists(connectionsPath)) 
			{
				XmlUtility xmlUtility = new XmlUtility();
				ArrayList connectionsList = xmlUtility.GetPropsFromXmlFile(connectionsPath);
				if(connectionsList == null || connectionsList.Count == 0) 
				{
					String errorMessage = "Error in " + spConnections;
					context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
				}
				return connectionsList;
			}
			else
			{
				return null;
			}
		}

		public bool IsReusable {
			get { return true; }
		}
	}
}